@@ -2,14 +2,6 @@
 Cross Section Regression 截面数据回归
 '''
 
-from math import e
-from numpy.core.defchararray import add
-
-import prettytable
-from statsmodels.genmod.generalized_estimating_equations import ParameterConstraint
-from .time_series_regress import TS_regress
-
-
 class CS_regress() :
     def __init__(self, list_y, factor) :
         import numpy as np
