@@ -8,27 +8,11 @@ which is divided into 4 steps
 4. present the result
 '''
 
-from asyncio import constants
-from calendar import c
-from msilib.schema import Error
-from sqlite3 import paramstyle
-from turtle import shape
-from unicodedata import decimal
-from unittest import expectedFailure
-
-from click import group
-from numpy import average, dtype
-from pyrsistent import v
-from regex import B
-from sklearn.model_selection import GroupShuffleSplit
-from typesentry import I
-
-
 class ptf_analysis():
     def __init__(self) :
         pass
     
-    def select_breakpoints(self,character,number,perc=None):
+    def select_breakpoints(self, character, number, perc=None, percn=None):
         '''
         select the breakpoints of the sample
         input : 
@@ -45,10 +29,14 @@ class ptf_analysis():
         # create percentiles
         if perc == None: 
             perc = np.linspace(0, 100, number+2, dtype=int)
-        elif perc is not None :
+        elif perc is not None:
             perc = perc
-            
-        breakpoint = np.percentile(character, perc,axis=0)
+
+        if percn is None:    
+            breakpoint = np.percentile(character, perc, axis=0)
+        elif percn is not None :
+            breakpoint = np.array(percn)
+
         return breakpoint
     
     def distribute(self, character, breakpoint):
@@ -142,10 +130,57 @@ class ptf_analysis():
                     average_statistics[i, j] = func(variable[np.where(label==temp_label[i]), j])
 
             return average_statistics
-
     
+    def create_breakpoint(self, data, number, perc=None):
+        '''
+        input: 
+            data(ndarray/DataFrame) : data having structure
+                first row to last two row: character
+                last row : time index
+            number (int) : the number of breakpoints
+            perc (array) : the percentiles of breakpoints
+        output:
+            breakpoints of characters
+        '''
+        import numpy as np
+       
+        if type(data).__name__ == 'DataFrame':
+            data = np.array(data)
+        
+        r, c =np.shape(data)
+
+        def divide_by_time(data):        
+            time = np.sort(np.unique(data[:, -1]))
+            groups_by_time = list()
+            for i in range(len(time)):
+                groups_by_time.append(data[np.where(data[:, -1]==time[i])])
+        
+            return groups_by_time
+        
+        # get the sample groups by time 得到按时间分组的样本
+        groups_time = divide_by_time(data)
+        # generate table of  average return for group and time 生成组和时间的表格
+        # Rows: groups 行： 组  
+        # Columns: Time 列： 时间
+        if perc is not None:
+            number = len(perc) - 2 
+        
+        breakpoint_array = list()
+        for i in range(c-1):
+            breakpoint_array.append(np.zeros((len(groups_time), number+2)))
+        
+        for i in range(len(groups_time)):
+            # for each time, a group exist
+            group = groups_time[i]
+            # for each time, generate breakpoint
+            for j in range(c-1):
+                breakpoint_array[j][i, :] = self.select_breakpoints(character = group[:, j], number = number, perc = perc)
+        
+        return breakpoint_array
+
+
 class Univariate(ptf_analysis):
-    def __init__(self, sample, number, perc=None, maxlag=12, weight=False):
+    def __init__(self, sample):
         '''
         input: sample: the samples to be analyzed 
             sample usually contain the future return, characteristics, time 
@@ -169,16 +204,7 @@ class Univariate(ptf_analysis):
             self._sample_type = 'ndarray'
         else:
             IOError
-        
-        self.number = number
-        if perc is not None :
-            self.number = len(perc) - 2
-        self.perc = perc
-        self.maxlag = maxlag
-        self._factor = None
-        self._time = np.sort(np.unique(self.sample[:, 2])) 
-        self._weight = weight       
-        
+             
     def divide_by_time(self, sample):
         '''
         split the sample by time into groups  将样本按照时间分组
@@ -198,7 +224,6 @@ class Univariate(ptf_analysis):
         average of the groups at each time point
         '''
         import numpy as np
-
         # get the sample groups by time 得到按时间分组的样本
         groups_time = self.divide_by_time(self.sample)
         # generate table of  average return for group and time 生成组和时间的表格
@@ -210,7 +235,10 @@ class Univariate(ptf_analysis):
             # for each time, a group exist
             group = groups_time[i]
             # for each time, generate breakpoint
-            breakpoint = super().select_breakpoints(group[:, 1], self.number, self.perc)
+            if self.percn is None:
+                breakpoint = super().select_breakpoints(character = group[:, 1], number = self.number, perc = self.perc)
+            elif self.percn is not None:
+                breakpoint = super().select_breakpoints(character = group[:, 1], number = self.number, percn = self.percn[i, :])
             # for each time, generate label
             label = super().distribute(group[:, 1], breakpoint)
             # for each group in each time, calculate the average future return
@@ -279,10 +307,27 @@ class Univariate(ptf_analysis):
 
         return self.average, self.ttest
     
-    def fit(self) :
+    def fit(self, number, perc=None, percn=None, maxlag=12, weight=False) :
         '''
         fit the model
         '''
+        import numpy as np
+
+        if perc is not None :
+            self.number = len(perc) - 2
+        elif percn is not None:
+            r, c = np.shape(percn)
+            self.number = c - 2
+        else:
+            self.number = number
+
+        self.perc = perc
+        self.percn = percn
+        self.maxlag = maxlag
+        self._factor = None
+        self._time = np.sort(np.unique(self.sample[:, 2])) 
+        self._weight = weight
+
         self.summary_and_test()
 
     def factor_adjustment(self, factor):
@@ -347,6 +392,28 @@ class Univariate(ptf_analysis):
 
         return alpha, ttest    
     
+    def extractor(self, pos):
+        '''
+        This function extract the return series
+        input:
+            pos (int): the position of the return series 
+        '''
+        import numpy as np
+        import pandas as pd
+        
+        if type(pos).__name__ != 'int':        
+            return IOError 
+
+        time = np.sort(np.unique(self.sample[:, 2]))
+        series_ex = self.result[pos, :]
+        r, c = np.shape(self.result)
+        if pos in [r-1, -1]:
+            series_ex = pd.DataFrame(np.array([time, series_ex]).T, columns=['Time', 'Diff'])
+        else:
+            series_ex = pd.DataFrame(np.array([time, series_ex]).T, columns=['Time', 'Ret'+str(pos)])
+
+        return series_ex
+
     def summary_statistics(self, variables=None, periodic=False):
         '''
         Summary Statistics 描述性统计
@@ -711,19 +778,24 @@ class Univariate(ptf_analysis):
             
             return df
         
-    def print_summary(self, explicit = False, export=False):
+    def print_summary(self, explicit = False, export=False, percentage=False):
         '''
         print summary
         '''
         import numpy as np
         from prettytable import PrettyTable
-
         # generate Table if no factor
         table = PrettyTable()
         table.add_column('Group', ['Average', 'T-Test'])
         for i in range(self.number+1):
-            table.add_column(str(i+1), np.around([self.average[i], self.ttest[0][i]], decimals=3))
-        table.add_column('Diff', np.around([self.average[-1], self.ttest[0][-1]], decimals=3))
+            if percentage == False:
+                table.add_column(str(i+1), np.around([self.average[i], self.ttest[0][i]], decimals=3))
+            elif percentage == True:
+                table.add_column(str(i+1), np.around([self.average[i]*100, self.ttest[0][i]], decimals=3))
+        if percentage == False:
+            table.add_column('Diff', np.around([self.average[-1], self.ttest[0][-1]], decimals=3))
+        elif percentage == True:
+            table.add_column('Diff', np.around([self.average[-1]*100, self.ttest[0][-1]], decimals=3))
         
         if self._factor is not None :
             table = PrettyTable()
@@ -753,22 +825,36 @@ class Univariate(ptf_analysis):
                 fac_re.append(self.r2[i][0])
 
                 if explicit == False:
-                    table.add_column(str(i+1), np.around([self.average[i], self.ttest[0][i], self.alpha[i][0], self.alpha_tvalue[i][0]], decimals=3))
+                    if percentage == False:
+                        table.add_column(str(i+1), np.around([self.average[i], self.ttest[0][i], self.alpha[i][0], self.alpha_tvalue[i][0]], decimals=3))
+                    elif percentage == True:
+                        table.add_column(str(i+1), np.around([self.average[i]*100, self.ttest[0][i], self.alpha[i][0]*100, self.alpha_tvalue[i][0]], decimals=3))
                 elif explicit == True:
-                    temp_re = [self.average[i], self.ttest[0][i], self.alpha[i][0], self.alpha_tvalue[i][0]] + fac_re
+                    if percentage == False:
+                        temp_re = [self.average[i], self.ttest[0][i], self.alpha[i][0], self.alpha_tvalue[i][0]] + fac_re
+                    elif percentage == True:
+                        temp_re = [self.average[i]*100, self.ttest[0][i], self.alpha[i][0]*100, self.alpha_tvalue[i][0]] + fac_re
                     table.add_column(str(i+1), np.around(temp_re, decimals=3))
             
             fac_re = list()
             for j in range(col_fac):
-                fac_re.append(self.beta[-1][j])
+                if percentage == False:
+                    fac_re.append(self.beta[-1][j])
+                elif percentage == True:
+                    fac_re.append(self.beta[-1][j]*100)
                 fac_re.append(self.beta_ttest[-1][j])
             fac_re.append(self.r2[-1][0])
 
             if explicit == False:
-                table.add_column('Diff', np.around([self.average[-1], self.ttest[0][-1], self.alpha[-1][0], self.alpha_tvalue[-1][0]], decimals=3))
+                if percentage == False:
+                    table.add_column('Diff', np.around([self.average[-1], self.ttest[0][-1], self.alpha[-1][0], self.alpha_tvalue[-1][0]], decimals=3))
+                elif percentage == True:
+                    table.add_column('Diff', np.around([self.average[-1]*100, self.ttest[0][-1], self.alpha[-1][0]*100, self.alpha_tvalue[-1][0]], decimals=3))
             elif explicit == True:
-                table.add_column('Diff', np.around([self.average[-1], self.ttest[0][-1], self.alpha[-1][0], self.alpha_tvalue[-1][0]] + fac_re, decimals=3))
-
+                if percentage == False:
+                    table.add_column('Diff', np.around([self.average[-1], self.ttest[0][-1], self.alpha[-1][0], self.alpha_tvalue[-1][0]] + fac_re, decimals=3))
+                elif percentage == True:
+                    table.add_column('Diff', np.around([self.average[-1]*100, self.ttest[0][-1], self.alpha[-1][0]*100, self.alpha_tvalue[-1][0]] + fac_re, decimals=3))
         np.set_printoptions(formatter={'float':'{:0.3f}'.format})
         print(table)
 
@@ -787,7 +873,7 @@ class Univariate(ptf_analysis):
 
     
 class Bivariate(ptf_analysis):
-    def __init__(self, sample, number=5, perc_row=None, perc_col=None, weight=False, maxlag=12):
+    def __init__(self, sample):
         '''
         input: sample: the samples to be analyzed 
             sample usually contain the future return, characteristics, time 
@@ -813,18 +899,7 @@ class Bivariate(ptf_analysis):
             self._sample_type = 'ndarray'
         else:
             IOError
-
-        self.number = number
-        self._factor = None
-        self.maxlag = maxlag
-        self.weight = weight
-        self.perc_row = perc_row
-        self.perc_col = perc_col
-        if (perc_row is not None) and (perc_col is not None):
-            self.perc_sign = True
-        else:
-            self.perc_sign = False
-
+        
         
     def divide_by_time(self):
         '''
@@ -845,33 +920,39 @@ class Bivariate(ptf_analysis):
         average of the groups at each time point
         '''
         import numpy as np
-
         # get the sample groups by time 得到按时间分组的样本
         groups_time = self.divide_by_time()
         # generate table of  average return for group and time 生成组和时间的表格
         # Rows: groups 行： 组  
         # Columns: Time 列： 时间
-        if self.perc_sign == False:
-            average_group_time = np.zeros((self.number+1, self.number+1, len(groups_time)))
-        elif self.perc_sign == True :
-            average_group_time = np.zeros((len(self.perc_row)-1, len(self.perc_col)-1, len(groups_time)))       
-        
+        average_group_time = np.zeros((self.num_row, self.num_col, len(groups_time)))
+                                      
         for i in range(len(groups_time)):
             # for each time, there exists a group 
             group = groups_time[i]
             # for each time, generate breakpoint
-            breakpoint_row = super().select_breakpoints(group[:, 1], self.number, self.perc_row)
+            if self.percn_row is None:
+                breakpoint_row = super().select_breakpoints(group[:, 1], self.num_row - 1, perc=self.perc_row)
+            else:
+                breakpoint_row = super().select_breakpoints(group[:, 1], self.num_row - 1, percn=self.percn_row[i, :])
             # for each time, generate label
             label_row = super().distribute(group[:, 1], breakpoint_row)[:, 0]
             
             if conditional == False:
-                breakpoint_col = super().select_breakpoints(group[:, 2], self.number, self.perc_col)
+                if self.percn_col is None:
+                    breakpoint_col = super().select_breakpoints(group[:, 2], self.num_col - 1, perc=self.perc_col)
+                else:
+                    breakpoint_col = super().select_breakpoints(group[:, 2], self.num_col - 1, percn=self.percn_col[i, :])
                 label_col = super().distribute(group[:, 2], breakpoint_col)[:, 0]
             elif conditional == True:
                 label_row_unique = list(np.unique(label_row))
                 label_col = - np.ones(len(group[:, 2]))
                 for j in range(len(label_row_unique)):
-                    breakpoint_col = super().select_breakpoints(group[:, 2][np.where(label_row==label_row_unique[j])], self.number, self.perc_col)
+                    if self.percn_col is None:
+                        breakpoint_col = super().select_breakpoints(group[:, 2][np.where(label_row==label_row_unique[j])], self.num_col - 1, perc=self.perc_col)
+                    else:
+                        breakpoint_col = super().select_breakpoints(group[:, 2][np.where(label_row==label_row_unique[j])], self.num_col - 1, percn=self.percn_col[i, :])
+
                     label_col[np.where(label_row==label_row_unique[j])] = super().distribute(group[:, 2][np.where(label_row==label_row_unique[j])], breakpoint_col)[:, 0]
 
             # for each group in each time, calculate the average future return
@@ -880,25 +961,25 @@ class Bivariate(ptf_analysis):
             if self.perc_sign == False:
                 if self.weight == False:
                     try:
-                        average_group_time[:,:,i] = super().average(group[:, 0], label, cond='bi').reshape((self.number+1, self.number+1))
+                        average_group_time[:,:,i] = super().average(group[:, 0], label, cond='bi').reshape((self.num_row, self.num_col))
                     except:
-                        average_group_time[:,:,i] = np.full([self.number+1, self.number+1], np.nan)
+                        average_group_time[:,:,i] = np.full([self.num_row, self.num_col], np.nan)
                 else:
                     try:
-                        average_group_time[:,:,i] = super().average(group[:, 0], label, cond='bi', weight=group[:, -1]).reshape((self.number+1, self.number+1))
+                        average_group_time[:,:,i] = super().average(group[:, 0], label, cond='bi', weight=group[:, 4]).reshape((self.num_row, self.num_col))
                     except:
-                        average_group_time[:,:,i] = np.full([self.number+1, self.number+1], np.nan)
+                        average_group_time[:,:,i] = np.full([self.num_row, self.num_col], np.nan)
             elif self.perc_sign == True:
                 if self.weight == False:
                     try:
-                        average_group_time[:,:,i] = super().average(group[:, 0], label, cond='bi').reshape((len(self.perc_row)-1, len(self.perc_col)-1))
+                        average_group_time[:,:,i] = super().average(group[:, 0], label, cond='bi').reshape((self.num_row, self.num_col))
                     except:
-                        average_group_time[:,:,i] = np.full([len(self.perc_row)-1, len(self.perc_col)-1], np.nan)
+                        average_group_time[:,:,i] = np.full([self.num_row, self.num_col], np.nan)
                 else:
                     try:
-                        average_group_time[:,:,i] = super().average(group[:, 0], label, cond='bi', weight=group[:, -1]).reshape((len(self.perc_row)-1, len(self.perc_col)-1))
+                        average_group_time[:,:,i] = super().average(group[:, 0], label, cond='bi', weight=group[:, 4]).reshape((self.num_row, self.num_col))
                     except:
-                        average_group_time[:,:,i] = np.full([len(self.perc_row)-1, len(self.perc_col)-1], np.nan)
+                        average_group_time[:,:,i] = np.full([self.num_row, self.num_col], np.nan)
 
         # return the Table
         # Rows: groups in each time
@@ -959,7 +1040,6 @@ class Bivariate(ptf_analysis):
                 # fit the model with the Newey-West Adjusment
                 # lags=maxlag
                 #re = model.fit()
-                print(temp_mat)
                 model = sm.OLS(temp_mat.iloc[:, 0], sm.add_constant(temp_mat.iloc[:, 1:]))
                 re = model.fit()
 
@@ -1025,9 +1105,65 @@ class Bivariate(ptf_analysis):
 
         return self.average, self.ttest
 
-    def fit(self, **kwargs):
+    def fit(self, number=4, perc_row=None, perc_col=None, percn_row=None, percn_col=None, weight=False, maxlag=12, **kwargs):
+        import numpy as np
+        # the number of groups
+
+        self._factor = None
+        self.maxlag = maxlag
+        self.weight = weight
+        self.perc_row = perc_row
+        self.perc_col = perc_col
+        self.percn_row = percn_row
+        self.percn_col = percn_col
+        if all([perc_row is None, perc_col is None, percn_row is None, percn_col is None]):
+            self.num_row = number + 1
+            self.num_col = number + 1
+            self.perc_sign = False
+        else:
+            self.perc_sign = True
+            if all([perc_row is not None, perc_col is not None]):
+                self.num_row = len(perc_row) - 1 
+                self.num_col = len(perc_col) - 1
+            elif all([percn_row is not None, percn_col is not None]):
+                r_row, c_row = np.shape(percn_row)
+                r_col, c_col = np.shape(percn_col)
+                self.num_row = c_row - 1
+                self.num_col = c_col - 1
+
         self.summary_and_test(**kwargs)
     
+    def extractor(self, r_pos, c_pos):
+        '''
+        This function extract the return series
+        input:
+            pos (int): the position of the return series 
+        '''
+        import numpy as np
+        import pandas as pd
+        
+        if type(r_pos).__name__ != 'int':        
+            return IOError 
+        elif type(c_pos).__name__ != 'int':
+            return IOError
+            
+        time = np.sort(np.unique(self.sample[:, 3]))
+        series_ex = self.result[r_pos, c_pos, :]
+        r, c, z = np.shape(self.result)
+        if r_pos in [r-1, -1] :
+            if c_pos in [c-1, -1]:
+                series_ex = pd.DataFrame(np.array([time, series_ex]).T, columns=['Time', 'Diff'+'Diff'])
+            else:
+                series_ex = pd.DataFrame(np.array([time, series_ex]).T, columns=['Time', 'Diff'+'Ret'+str(c_pos)])            
+        else:
+            if c_pos in [c-1, -1]:
+                series_ex = pd.DataFrame(np.array([time, series_ex]).T, columns=['Time', 'Ret'+str(r_pos)+'Diff'])
+            else:
+                series_ex = pd.DataFrame(np.array([time, series_ex]).T, columns=['Time', 'Ret'+str(r_pos)+'Ret'+str(c_pos)])
+
+        return series_ex
+
+
     def print_summary_by_time(self, export=False) :
         '''
         print summary_by_time
@@ -1038,7 +1174,7 @@ class Bivariate(ptf_analysis):
         r, c, n = np.shape(self.result)
         table = PrettyTable()
         time = np.sort(np.unique(self.sample[:, 3]))
-        table.field_names = ['Time', 'Group'] + [str(i+1) for i in range(self.number+1)] + ['Diff']
+        table.field_names = ['Time', 'Group'] + [str(i+1) for i in range(self.num_row)] + ['Diff']
         for i in range(n):
             for j in range(r):
                 if j == 0 :
@@ -1066,23 +1202,22 @@ class Bivariate(ptf_analysis):
             
             return df
         
-    def print_summary(self, explicit=False, export=False):
+    def print_summary(self, explicit=False, export=False, percentage=False):
         '''
         print summary
         '''
         import numpy as np
         from prettytable import PrettyTable
-        
         # generate Table if no factor
         if self._factor is None :
             table=PrettyTable()
             if self._sample_type == 'ndarray':
-                table.field_names = ['Group'] + [i+1 for i in range(self.number+1)] + ['Diff']
+                table.field_names = ['Group'] + [i+1 for i in range(self.num_col)] + ['Diff']
             elif self._sample_type == 'DataFrame':
-                table.field_names = ['Group'] + [self._columns[2] + str(i+1) for i in range(self.number+1)] + ['Diff']
+                table.field_names = ['Group'] + [self._columns[2] + str(i+1) for i in range(self.num_col)] + ['Diff']
 
-            for i in range(self.number+2):
-                if i == self.number+1 :
+            for i in range(self.num_row+1):
+                if i == self.num_row :
                     temp = ['Diff']
                     temp_tvalue = [' ']
                 else:
@@ -1091,8 +1226,12 @@ class Bivariate(ptf_analysis):
                     elif self._sample_type == 'DataFrame':
                         temp = [self._columns[1] + str(i+1)]
                     temp_tvalue = [' ']
-                for j in range(self.number+2):
-                    temp.append(np.around(self.average[i, j], decimals=3))
+                for j in range(self.num_col+1):
+                    if percentage == False:
+                        temp.append(np.around(self.average[i, j], decimals=3))
+                    elif percentage == True:
+                        temp.append(np.around(self.average[i, j]*100, decimals=3))
+                    
                     temp_tvalue.append(np.around(self.ttest[0][i, j], decimals=3))
 
                 table.add_row(temp)
@@ -1102,17 +1241,17 @@ class Bivariate(ptf_analysis):
             table = PrettyTable()
             row_fac, col_fac = np.shape(self._factor)
             if self._sample_type == 'ndarray':
-                table.field_names = ['Group'] + [i+1 for i in range(self.number+1)] + ['Diff']
+                table.field_names = ['Group'] + [i+1 for i in range(self.num_col)] + ['Diff']
             elif self._sample_type == 'DataFrame':
-                table.field_names = ['Group'] + [self._columns[2] + str(i+1) for i in range(self.number+1)] + ['Diff']
+                table.field_names = ['Group'] + [self._columns[2] + str(i+1) for i in range(self.num_col)] + ['Diff']
 
             if type(self._factor).__name__ == 'DataFrame':
                 fac_name = self._factor.columns
             elif type(self._factor).__name__ == 'ndarray':
                 fac_name = ['factor' + str(i+1) for i in range(col_fac)]
 
-            for i in range(self.number+2):
-                if i == self.number+1:
+            for i in range(self.num_row+1):
+                if i == self.num_row:
                     temp = ['Diff']
                     temp_tvalue = [' ']
                     temp_fac = ['alpha']
@@ -1127,10 +1266,16 @@ class Bivariate(ptf_analysis):
                     temp_fac = ['alpha']
                     temp_fac_tvalue = [' ']
                     temp_r2 = ['R2']
-                for j in range(self.number+2):
-                    temp.append(np.around(self.average[i, j], decimals=3))
+                for j in range(self.num_col+1):
+                    if percentage == False:
+                        temp.append(np.around(self.average[i, j], decimals=3))
+                    elif percentage == True:
+                        temp.append(np.around(self.average[i, j]*100, decimals=3))
                     temp_tvalue.append(np.around(self.ttest[0][i, j], decimals=3))
-                    temp_fac.append(np.around(self.alpha[i, j], decimals=3))
+                    if percentage == False:
+                        temp_fac.append(np.around(self.alpha[i, j], decimals=3))
+                    elif percentage == True:
+                        temp_fac.append(np.around(self.alpha[i, j]*100, decimals=3))
                     temp_fac_tvalue.append(np.around(self.alpha_tvalue[i, j], decimals=3))
                     temp_r2.append(np.around(self.r2[i, j], decimals=3))
 
@@ -1144,8 +1289,12 @@ class Bivariate(ptf_analysis):
                         temp_fac_beta = [fac_name[k]]
                         temp_fac_betatvalue = ['t-value']
                         temp_blank = [' ']
-                        for j in range(self.number+2):
-                            temp_fac_beta.append(np.around(self.beta[i, j, k], decimals=3))
+                        for j in range(self.num_col+1):
+                            if percentage == False:
+                                temp_fac_beta.append(np.around(self.beta[i, j, k], decimals=3))
+                            elif percentage == True:
+                                temp_fac_beta.append(np.around(self.beta[i, j, k]*100, decimals=3))
+                            
                             temp_fac_betatvalue.append(np.around(self.beta_test[i, j, k], decimals=3))
                             temp_blank.append(' ')
                     
@@ -1505,3 +1654,4 @@ class Spanning_test():
         
         table.float_format = '.5'
         print(table)
+
