@@ -3,12 +3,7 @@ Time Series Analysis 时间序列分析
 
 '''
 
-from ast import Constant
 from distutils.log import ERROR
-from random import sample
-from numpy.core.defchararray import add
-from statsmodels.multivariate import factor
-
 
 class TS_regress() :
     def __init__(self, list_y, factor) :
